import numpy as np

from pymop.problem import Problem


class Schwefel(Problem):
    def __init__(self, n_var=2):
        Problem.__init__(self)
        self.n_var = n_var
        self.n_constr = 0
        self.n_obj = 1
        self.func = self._evaluate
        self.xl = -500 * np.ones(self.n_var)
        self.xu = 500 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        f[:, 0] = 418.9829 * self.n_var - np.sum(x * np.sin(np.sqrt(np.abs(x))), axis=1)



import numpy as np


def get_uniform_weights(n_points, n_dim):
    return UniformReferenceDirectionFactory(n_dim, n_points=n_points).do()


def binomial(n, k):
    if 0 <= k <= n:
        ntok = 1
        ktok = 1
        for t in range(1, min(k, n - k) + 1):
            ntok *= n
            ktok *= t
            n -= 1
        return ntok // ktok
    else:
        return 0


class UniformReferenceDirectionFactory:

    def __init__(self, n_dim, n_points=None, n_partitions=None) -> None:

        self.n_dim = n_dim
        if n_points is not None:
            self.n_partitions = UniformReferenceDirectionFactory.get_partition_closest_to_points(n_points, n_dim)
        else:
            if n_partitions is None:
                raise Exception("Either provide number of partitions or number of points.")
            else:
                self.n_partitions = n_partitions

    def do(self):
        return self.uniform_reference_directions(self.n_partitions, self.n_dim)

    @staticmethod
    def get_partition_closest_to_points(n_points, n_dim):

        # in this case the do method will always return one values anyway
        if n_dim == 1:
            return 0

        n_partitions = 1
        _n_points = UniformReferenceDirectionFactory.get_n_points(n_partitions, n_dim)
        while _n_points <= n_points:
            n_partitions += 1
            _n_points = UniformReferenceDirectionFactory.get_n_points(n_partitions, n_dim)
        return n_partitions - 1

    @staticmethod
    def get_n_points(n_partitions, n_dim):
        return int(binomial(n_dim + n_partitions - 1, n_partitions))

    def uniform_reference_directions(self, n_partitions, n_dim):
        ref_dirs = []
        ref_dir = np.full(n_dim, np.inf)
        self.__uniform_reference_directions(ref_dirs, ref_dir, n_partitions, n_partitions, 0)
        return np.concatenate(ref_dirs, axis=0)

    def __uniform_reference_directions(self, ref_dirs, ref_dir, n_partitions, beta, depth):
        if depth == len(ref_dir) - 1:
            ref_dir[depth] = beta / (1.0 * n_partitions)
            ref_dirs.append(ref_dir[None, :])
        else:
            for i in range(beta + 1):
                ref_dir[depth] = 1.0 * i / (1.0 * n_partitions)
                self.__uniform_reference_directions(ref_dirs, np.copy(ref_dir), n_partitions, beta - i,
                                                    depth + 1)


if __name__ == "__main__":
    w = get_uniform_weights(200, 3)
    # w = UniformReferenceDirectionFactory(3, n_points=100).do()

    import matplotlib.pyplot as plt

    from pymop import *

    w = DTLZ5(10, 3, n_pareto_points=91).pareto_front()

    fig = plt.figure()
    from mpl_toolkits.mplot3d import Axes3D

    ax = fig.add_subplot(111, projection='3d')
    ax.scatter(w[:, 0], w[:, 1], w[:, 2])
    plt.show()





def evaluate():
    import numpy as np

    # initialize it with the necessary parameters
    from pymop.problems.dtlz import DTLZ1
    problem = DTLZ1(n_var=10, n_obj=3)

    # evaluation function returns by default two numpy arrays - objective function values and constraints -
    # as input either provide a vector
    F, G = problem.evaluate(np.random.random(10))

    # or a whole matrix to evaluate several solutions at once
    F, G = problem.evaluate(np.random.random((100, 10)))

    # if no constraints should be returned
    F = problem.evaluate(np.random.random((100, 10)), return_constraint_violation=False)

    from pymop.problems.welded_beam import WeldedBeam
    F, CV = WeldedBeam().evaluate(np.random.random((100, 4)), return_constraint_violation=True)


def plot():
    from pymop import plot_problem_surface, Ackley
    plot_problem_surface(Ackley(n_var=1), 200)
    plot_problem_surface(Ackley(n_var=2), 200, plot_type="wireframe")
    plot_problem_surface(Ackley(n_var=2), 200, plot_type="contour")



import numpy as np

from pymop.problem import Problem
from pymop.util import get_uniform_weights


class DTLZ(Problem):
    def __init__(self, n_var, n_obj, n_pareto_points=None):
        Problem.__init__(self)
        self.n_obj = n_obj
        self.n_var = n_var
        self.n_constr = 0
        self.func = self._evaluate
        self.xl = np.zeros(self.n_var)
        self.xu = np.ones(self.n_var)

        self.k = self.n_var - self.n_obj + 1
        self.n_pareto_points = n_pareto_points

    def g1(self, X_M):
        return 100 * (self.k + np.sum(np.square(X_M - 0.5) - np.cos(20 * np.pi * (X_M - 0.5)), axis=1))

    def g2(self, X_M):
        return np.sum(np.square(X_M - 0.5), axis=1)

    def obj_func(self, X_, g, f, alpha=1):
        for i in range(0, self.n_obj):
            f[:, i] = (1 + g)
            f[:, i] *= np.prod(np.cos(np.power(X_[:, :X_.shape[1] - i], alpha) * np.pi / 2.0), axis=1)
            if i > 0:
                f[:, i] *= np.sin(np.power(X_[:, X_.shape[1] - i], alpha) * np.pi / 2.0)

    def pareto_front(self):
        if self.n_pareto_points is None:
            raise Exception("Please specify how many pareto-optimal points are desired by setting n_pareto_points!")

        return super().pareto_front()


def generic_sphere(n_points, n_dim):
    w = get_uniform_weights(n_points, n_dim)
    F = w / np.tile(np.linalg.norm(w, axis=1)[:, None], (1, w.shape[1]))
    return F


class DTLZ1(DTLZ):
    def __init__(self, n_var=10, n_obj=3, **kwargs):
        super().__init__(n_var, n_obj, **kwargs)

    def _calc_pareto_front(self):
        return 0.5 * get_uniform_weights(self.n_pareto_points, self.n_obj)

    def _evaluate(self, x, f):
        X_, X_M = x[:, :self.n_obj - 1], x[:, self.n_obj - 1:]
        g = self.g1(X_M)
        for i in range(0, self.n_obj):
            f[:, i] = 0.5 * (1 + g)
            f[:, i] *= np.prod(X_[:, :X_.shape[1] - i], axis=1)
            if i > 0:
                f[:, i] *= 1 - X_[:, X_.shape[1] - i]


class DTLZ2(DTLZ):
    def __init__(self, n_var=10, n_obj=3, **kwargs):
        super().__init__(n_var, n_obj, **kwargs)

    def _calc_pareto_front(self):
        return generic_sphere(self.n_pareto_points, self.n_obj)

    def _evaluate(self, x, f):
        X_, X_M = x[:, :self.n_obj - 1], x[:, self.n_obj - 1:]
        g = self.g2(X_M)
        self.obj_func(X_, g, f, alpha=1)


class DTLZ3(DTLZ):
    def __init__(self, n_var=10, n_obj=3, **kwargs):
        super().__init__(n_var, n_obj, **kwargs)

    def _calc_pareto_front(self):
        return generic_sphere(self.n_pareto_points, self.n_obj)

    def _evaluate(self, x, f):
        X_, X_M = x[:, :self.n_obj - 1], x[:, self.n_obj - 1:]
        g = self.g1(X_M)
        self.obj_func(X_, g, f, alpha=1)


class DTLZ4(DTLZ):
    def __init__(self, n_var=10, n_obj=3, alpha=100, d=100, **kwargs):
        super().__init__(n_var, n_obj, **kwargs)
        self.alpha = alpha
        self.d = d

    def _calc_pareto_front(self):
        return generic_sphere(self.n_pareto_points, self.n_obj)

    def _evaluate(self, x, f):
        X_, X_M = x[:, :self.n_obj - 1], x[:, self.n_obj - 1:]
        g = self.g2(X_M)
        self.obj_func(X_, g, f, alpha=self.alpha)


class DTLZ5(DTLZ):
    def __init__(self, n_var=10, n_obj=3, **kwargs):
        super().__init__(n_var, n_obj, **kwargs)

    def _calc_pareto_front(self):
        pass

    def _evaluate(self, x, f):
        X_, X_M = x[:, :self.n_obj - 1], x[:, self.n_obj - 1:]
        g = self.g2(X_M)

        theta = 1 / (2 * (1 + g[:, None])) * (1 + 2 * g[:, None] * X_)
        theta[:, 0] = x[:, 0]
        self.obj_func(theta, g, f)


class DTLZ6(DTLZ):
    def __init__(self, n_var=10, n_obj=3, **kwargs):
        super().__init__(n_var, n_obj, **kwargs)

    def _calc_pareto_front(self):
        pass

    def _evaluate(self, x, f):
        X_, X_M = x[:, :self.n_obj - 1], x[:, self.n_obj - 1:]
        g = np.sum(np.power(X_M, 0.1), axis=1)

        theta = 1 / (2 * (1 + g[:, None])) * (1 + 2 * g[:, None] * X_)
        theta[:, 0] = x[:, 0]
        self.obj_func(theta, g, f)


class DTLZ7(DTLZ):
    def __init__(self, n_var=10, n_obj=3, **kwargs):
        super().__init__(n_var, n_obj, **kwargs)

    def _evaluate(self, x, f):
        for i in range(0, self.n_obj - 1):
            f[:, i] = x[:, i]

        g = 1 + 9 / self.k * np.sum(x[:, -self.k:], axis=1)
        h = self.n_obj - np.sum(f[:, :-1] / (1 + g[:, None]) * (1 + np.sin(3 * np.pi * f[:, :-1])), axis=1)
        f[:, self.n_obj - 1] = (1 + g) * h






from pymop.problems.ackley import *
from pymop.problems.bnh import *
from pymop.problems.carside import *
from pymop.problems.dtlz import *
from pymop.problems.griewank import *
from pymop.problems.knapsack import *
from pymop.problems.kursawe import *
from pymop.problems.osy import *
from pymop.problems.rastrigin import *
from pymop.problems.rosenbrock import *
from pymop.problems.schwefel import *
from pymop.problems.sphere import *
from pymop.problems.tnk import *
from pymop.problems.trust_2d import *
from pymop.problems.welded_beam import *
from pymop.problems.zakharov import *
from pymop.problems.zdt import *


import os
import numpy as np
import inspect


class Problem:
    """
    Superclass for each problem that is defined. It provides attributes such
    as the number of variables, number of objectives or constraints.
    Also, the lower and upper bounds are stored. If available the Pareto-front, nadir point
    and ideal point are stored.
    """

    def __init__(self, n_var=0, n_obj=0, n_constr=0, xl=None, xu=None, func=None):
        """

        Parameters
        ----------
        n_var : int
            number of variables
        n_obj : int
            number of objectives
        n_constr : int
            number of constraints
        xl : np.ndarray
            lower bounds for the variables
        xu : np.ndarray
            upper bounds for the variable
        func : func
            function that evaluates the problem. Useful if a problem is defined inplace.
        """
        self.n_var = n_var
        self.n_obj = n_obj
        self.n_constr = n_constr
        self.xl = xl if type(xl) is np.ndarray else np.ones(n_var) * xl
        self.xu = xu if type(xu) is np.ndarray else np.ones(n_var) * xu
        self.func = func
        self._pareto_front = None

    # return the maximum objective values of the pareto front
    def nadir_point(self):
        """
        Returns
        -------
        nadir_point : np.ndarray
            The nadir point for a multi-objective problem.
            If single-objective, it returns the best possible solution which is equal to the ideal point.

        """
        return np.max(self.pareto_front(), axis=0)

    # return the minimum values of the pareto front
    def ideal_point(self):
        """
        Returns
        -------
        ideal_point : np.ndarray
            The ideal point for a multi-objective problem. If single-objective
            it returns the best possible solution.
        """
        return np.min(self.pareto_front(), axis=0)

    def pareto_front(self):
        """
        Returns
        -------
        P : np.ndarray
            The Pareto front of a given problem. It is only loaded or calculate the first time and then cached.
            For a single-objective problem only one point is returned but still in a two dimensional array.
        """
        if self._pareto_front is None:
            self._pareto_front = self._calc_pareto_front()

            if self._pareto_front is None:
                raise Exception("Pareto front for this tests problem not found.")

        return self._pareto_front

    def evaluate(self, X, return_constraint_violation=True, return_constraints=False, **kwargs):

        """
        Evaluate the given problem.

        The function values set as defined in the function.
        The constraint values are meant to be positive if infeasible. A higher positive values means "more" infeasible".
        If they are 0 or negative, they will be considered as feasible what ever their value is.

        Parameters
        ----------
        X : np.ndarray
            A two dimensional matrix where each row is a point to evaluate and each column a variable.

        return_constraint_violation : bool
            Whether the constraint violation is returned or not. If no constraint exists,
            an array with zero values is returned. Default: True.
            
        return_constraints : bool
            Whether all constraint values are returned or not. Default: False.

        Returns
        -------
        F : np.ndarray
            Objective Values
        CV : np.ndarray
            Constraint Violations as a one dimensional array.
        G : np.ndarray
            Constraints as a two dimensional array.

        """

        only_single_value = len(np.shape(X)) == 1
        if only_single_value:
            X = np.array([X])

        # check the dimensionality of the problem and the given input
        if X.shape[1] != self.n_var:
            raise Exception('Input dimension %s are not equal to n_var %s!' % (X.shape[1], self.n_var))

        # create the resulting arrays
        f = np.zeros((X.shape[0], self.n_obj))
        g = np.zeros((X.shape[0], self.n_constr))

        # if constraints exists func(x, f, g) is used otherwise just func(x, f)
        params = inspect.signature(self.func).parameters

        # optionally allow to have individuals also as a parameter and call the subfunction not in kwargs
        if 'individuals' in kwargs:
            individuals = kwargs.pop('individuals')
        else:
            individuals = None

        _kwargs = {}
        for key, value in kwargs.items():
            if key in params:
                _kwargs[key] = value

        args = [X, f]

        if self.n_constr > 0:
            args.append(g)
            if len(params) == 4:
                args.append(individuals)
        else:
            if len(params) == 3:
                args.append(individuals)

        self.func(*args, **_kwargs)

        # create the returned values in a list
        vals = [f]
        if return_constraint_violation:
            vals.append(Problem.calc_constraint_violation(g))
        if return_constraints:
            vals.append(g)

        # convert back if just one vector is evaluated
        if only_single_value:
            vals = [e[0, :] for e in vals]

        if len(vals) > 1:
            return tuple(vals)
        else:
            return vals[0]

    def name(self):
        """
        Returns
        -------
        name : str
            The name of the problem. Per default it is the name of the class but it can be overridden.
        """
        return self.__class__.__name__

    def _calc_pareto_front(self):
        """
        Default behaviour is look to look for the pareto front file.
        If this does not exist return None.

        Returns
        -------
        pf : numpy.array
            Pareto optimal front for the problem. In case of single-objective just one value

        """
        current_dir = os.path.dirname(os.path.realpath(__file__))
        fname = os.path.join(current_dir, "pf", "%s.pf" % self.name())
        if os.path.isfile(fname):
            return np.loadtxt(fname)
        return None

    # some problem information
    def __str__(self):
        s = "# name: %s\n" % self.name()
        s += "# n_var: %s\n" % self.n_var
        s += "# n_obj: %s\n" % self.n_obj
        s += "# n_constr: %s\n" % self.n_constr
        s += "# f(xl): %s\n" % self.evaluate(self.xl)[0]
        s += "# f((xl+xu)/2): %s\n" % self.evaluate((self.xl + self.xu) / 2.0)[0]
        s += "# f(xu): %s\n" % self.evaluate(self.xu)[0]
        return s

    @staticmethod
    def calc_constraint_violation(G):
        if G is None:
            return None
        elif G.shape[1] == 0:
            return np.zeros(G.shape[0])[:, None]
        else:
            return np.sum(G * (G > 0).astype(np.float), axis=1)[:, None]


class ScaledProblem(Problem):

    def __init__(self, problem, scale_factor):
        super().__init__(problem.n_var, problem.n_obj, problem.n_constr, problem.xl, problem.xu, problem.func)
        self.problem = problem
        self.scale_factor = scale_factor

    @staticmethod
    def get_scale(n, scale_factor):
        return np.power(np.full(n, scale_factor), np.arange(n))

    def evaluate(self, X, **kwargs):
        t = self.problem.evaluate(X, **kwargs)
        F = t[0] * ScaledProblem.get_scale(self.n_obj, self.scale_factor)
        return tuple([F] + list(t)[1:])

    def _calc_pareto_front(self):
        return self.problem.pareto_front() * ScaledProblem.get_scale(self.n_obj, self.scale_factor)


class ConvexProblem(Problem):

    def __init__(self, problem):
        super().__init__(problem.n_var, problem.n_obj, problem.n_constr, problem.xl, problem.xu, problem.func)
        self.problem = problem

    @staticmethod
    def get_power(n):
        p = np.full(n, 4.0)
        p[-1] = 2.0
        return p

    def evaluate(self, X, **kwargs):
        t = self.problem.evaluate(X, **kwargs)
        F = np.power(t[0], ConvexProblem.get_power(self.n_obj))
        return tuple([F] + list(t)[1:])

    def _calc_pareto_front(self):
        F = self.problem.pareto_front()
        return np.power(F, ConvexProblem.get_power(self.n_obj))



import numpy as np

from pymop.problem import Problem


class Griewank(Problem):
    def __init__(self, n_var=2):
        Problem.__init__(self)
        self.n_var = n_var
        self.n_constr = 0
        self.n_obj = 1
        self.func = self._evaluate
        self.xl = -600 * np.ones(self.n_var)
        self.xu = 600 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        f[:,0] = 1 + 1 / 4000 * np.sum(np.power(x,2), axis=1) \
                                 - np.prod(np.cos(x / np.sqrt(np.arange(1,x.shape[1]+1))), axis=1)



import numpy as np

from pymop.problem import Problem


class TNK(Problem):
    def __init__(self):
        Problem.__init__(self)
        self.n_var = 2
        self.n_constr = 2
        self.n_obj = 2
        self.func = self._evaluate
        self.xl = np.array([0, 1e-30])
        self.xu = np.array([np.pi, np.pi])

    def _evaluate(self, x, f, g):
        f[:, 0] = x[:, 0]
        f[:, 1] = x[:, 1]
        g[:, 0] = -(np.square(x[:, 0]) + np.square(x[:, 1]) - 1.0 - 0.1 * np.cos(16.0 * np.arctan(x[:, 0] / x[:, 1])))
        g[:, 1] = 2 * (np.square(x[:, 0] - 0.5) + np.square(x[:, 1] - 0.5)) - 1



import numpy as np

from pymop.problem import Problem


class Rastrigin(Problem):
    def __init__(self, n_var=2, A=10.0):
        Problem.__init__(self)
        self.n_var = n_var
        self.A = A
        self.n_constr = 0
        self.n_obj = 1
        self.func = self._evaluate
        self.xl = -5 * np.ones(self.n_var)
        self.xu = 5 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        z = np.power(x, 2) - self.A * np.cos(2 * np.pi * x)
        f[:, 0] = self.A * self.n_var + np.sum(z, axis=1)



import numpy as np

from pymop.problem import Problem


class Zakharov(Problem):
    def __init__(self, n_var=2):
        Problem.__init__(self)
        self.n_var = n_var
        self.n_constr = 0
        self.n_obj = 1
        self.func = self._evaluate
        self.xl = -10 * np.ones(self.n_var)
        self.xu = 10 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        a = np.sum(0.5 * np.arange(1, self.n_var + 1) * x, axis=1)
        f[:, 0] = np.sum(np.square(x), axis=1) + np.square(a) + np.power(a, 4)





import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(__file__, "../..")))

import numpy as np

if __name__ == "__main__":

    if len(sys.argv) != 4:
        raise Exception("Usage: cmd.py <problem> <params> <func>")

    _, par_problem, par_params, par_func = sys.argv

    # imports all problem in this framework
    from pymop import *

    try:
        clazz = eval(par_problem)
    except Exception as e:
        print("ERROR: Problem %s not found." % par_problem[0])
        print(e)
        exit(1)

    # no parameter given - just "[]"
    if len(par_params) <= 2:
        par_params = []
    else:
        par_params = par_params.replace("[", "").replace("]", "").split(",")

    if len(par_params) % 2 != 0:
        raise Exception("Parameters must have an even number of arguments. [key, value, key, value]. Currently: %s" % len(par_params))

    try:

        params_of_constructor = clazz.__init__.__code__.co_varnames[1:]
        default_values_of_params = clazz.__init__.__defaults__

        # if no params are defined just try without any
        if len(params_of_constructor) == 0:
            problem = clazz()

        # else parse the params as dict
        else:

            if len(params_of_constructor) != len(default_values_of_params):
                raise Exception("Each problem used externally must have provided default values for all constructor arguments.")

            params = {}
            for i in range(int(len(par_params)/2)):
                key = par_params[2*i].strip()
                val = par_params[2*i+1].strip()

                if key in params_of_constructor:
                    default_type = type(default_values_of_params[params_of_constructor.index(key)])
                    params[key] = default_type(val)

            problem = clazz(**params)

    except Exception as e:
        print("ERROR: Problem can not be initialized with params %s." % params)
        print(e)
        exit(1)

    # execute the method - use stdin and stdout if necessary
    if par_func == "evaluate":

        X = np.loadtxt(sys.stdin.readlines(), dtype=np.float)

        if len(X.shape) == 1:
            X = X[None, :]

        F, G = problem.evaluate(X)
        if problem.n_constr > 0:
            F = np.concatenate([F, G], axis=1)

        np.savetxt(sys.stdout, F, fmt='%.18e', delimiter=' ')

    elif par_func == 'front':
        np.savetxt(par_out, problem._calc_pareto_front(), fmt='%.18e', delimiter=' ')

    elif par_func == 'info':
        import json

        d = {
            'n_var': problem.n_var,
            'n_obj': problem.n_obj,
            'n_constr': problem.n_constr,
            'xl': problem.xl.tolist(),
            'xu': problem.xu.tolist()
        }
        print(json.dumps(d))

    else:
        print("ERROR: Method does not exist.")
        exit(1)

    sys.stdout.flush()


import numpy as np

from pymop.problem import Problem


class ZDT(Problem):
    def __init__(self, n_var=30):
        Problem.__init__(self, func=self._evaluate)
        self.n_var = n_var
        self.n_constr = 0
        self.n_obj = 2

        self.xl = np.zeros(self.n_var)
        self.xu = np.ones(self.n_var)


class ZDT1(ZDT):
    def __init__(self, n_var=30):
        ZDT.__init__(self, n_var)

    def _calc_pareto_front(self):
        x1 = np.arange(0, 1.01, 0.01)
        return np.array([x1, 1 - np.sqrt(x1)]).T

    def _evaluate(self, x, f):
        f[:, 0] = x[:, 0]
        g = 1 + 9.0 / (self.n_var - 1) * np.sum(x[:, 1:], axis=1)
        f[:, 1] = g * (1 - np.power((f[:, 0] / g), 0.5))


class ZDT2(ZDT):
    def __init__(self, n_var=30):
        ZDT.__init__(self, n_var)

    def _calc_pareto_front(self):
        x1 = np.arange(0, 1.01, 0.01)
        return np.array([x1, 1 - np.power(x1, 2)]).T

    def _evaluate(self, x, f):
        f[:, 0] = x[:, 0]
        c = np.sum(x[:, 1:], axis=1)
        g = 1.0 + 9.0 * c / (self.n_var - 1)
        f[:, 1] = g * (1 - np.power((f[:, 0] * 1.0 / g), 2))


class ZDT3(ZDT):
    def __init__(self, n_var=30):
        ZDT.__init__(self, n_var)

    def _calc_pareto_front(self):
        regions = [[0, 0.0830015349], [0.182228780, 0.2577623634],
                   [0.4093136748, 0.4538821041], [0.6183967944, 0.6525117038],
                   [0.8233317983, 0.8518328654]]

        pareto_front = np.array([]).reshape((-1, 2))
        for r in regions:
            x1 = np.linspace(r[0], r[1], 50)
            x2 = 1 - np.sqrt(x1) - x1 * np.sin(10 * np.pi * x1)
            pareto_front = np.concatenate((pareto_front, np.array([x1, x2]).T), axis=0)
        return pareto_front

    def _evaluate(self, x, f):
        f[:, 0] = x[:, 0]
        c = np.sum(x[:, 1:], axis=1)
        g = 1.0 + 9.0 * c / (self.n_var - 1)
        f[:, 1] = g * (1 - np.power(f[:, 0] * 1.0 / g, 0.5) - (f[:, 0] * 1.0 / g) * np.sin(10 * np.pi * f[:, 0]))


class ZDT4(ZDT):
    def __init__(self, n_var=10):
        ZDT.__init__(self, n_var)
        self.xl = -5 * np.ones(self.n_var)
        self.xl[0] = 0.0
        self.xu = 5 * np.ones(self.n_var)
        self.xu[0] = 1.0
        self.func = self._evaluate

    def _calc_pareto_front(self):
        x1 = np.arange(0, 1.01, 0.01)
        return np.array([x1, 1 - np.sqrt(x1)]).T

    def _evaluate(self, x, f):
        f[:, 0] = x[:, 0]
        g = 1.0
        g += 10 * (self.n_var - 1)
        for i in range(1, self.n_var):
            g += x[:, i] * x[:, i] - 10.0 * np.cos(4.0 * np.pi * x[:, i])
        h = 1.0 - np.sqrt(f[:, 0] / g)
        f[:, 1] = g * h


class ZDT6(ZDT):
    def __init__(self, n_var=10):
        ZDT.__init__(self, n_var)
        self.func = self._evaluate

    def _calc_pareto_front(self):
        x1 = np.linspace(0.2807753191, 1, 100)
        return np.array([x1, 1 - np.power(x1, 2)]).T

    def _evaluate(self, x, f):
        f[:, 0] = 1 - np.exp(-4 * x[:, 0]) * np.power(np.sin(6 * np.pi * x[:, 0]), 6)
        g = 1 + 9.0 * np.power(np.sum(x[:, 1:], axis=1) / (self.n_var - 1.0), 0.25)
        f[:, 1] = g * (1 - np.power(f[:, 0] / g, 2))



import numpy as np


def plot_problem_surface(problem, n_samples, plot_type="wireframe"):
    try:
        import matplotlib.pyplot as plt
        from mpl_toolkits.mplot3d import Axes3D
    except:
        raise Exception("Please install 'matplotlib' to use the plotting functionality.")

    if problem.n_var == 1 and problem.n_obj == 1:

        X = np.linspace(problem.xl[0], problem.xu[0], num=n_samples)[:, None]
        Y = problem.evaluate(X, return_constraint_violation=False)
        plt.plot(X, Y)

    elif problem.n_var == 2 and problem.n_obj == 1:


        X_range = np.linspace(problem.xl[0], problem.xu[0], num=n_samples)
        Y_range = np.linspace(problem.xl[1], problem.xu[1], num=n_samples)
        X, Y = np.meshgrid(X_range, Y_range)

        A = np.zeros((n_samples * n_samples, 2))
        counter = 0
        for i, x in enumerate(X_range):
            for j, y in enumerate(Y_range):
                A[counter, 0] = x
                A[counter, 1] = y
                counter += 1

        F = np.reshape(problem.evaluate(A, return_constraint_violation=False), (n_samples, n_samples))

        fig = plt.figure()
        # Plot the surface.

        if plot_type == "wireframe":
            ax = fig.add_subplot(111, projection='3d')
            ax.plot_wireframe(X, Y, F)
        elif plot_type == "contour":
            CS = plt.contour(X, Y, F)
            plt.clabel(CS, inline=1, fontsize=10)
        else:
            raise Exception("Unknown plotting method.")


    else:
        raise Exception("Can only plot problems with less than two variables and one objective.")

    plt.show()



import numpy as np

from pymop.problem import Problem


class Knapsack(Problem):
    def __init__(self,
                 n_items,  # number of items that can be picked up
                 W,  # weights for each item
                 P,  # profit of each item
                 C,  # maximum capacity
                 ):
        Problem.__init__(self)
        self.n_var = n_items
        self.n_constr = 1
        self.n_obj = 1
        self.func = self._evaluate

        self.W = W
        self.P = P
        self.C = C

    def _evaluate(self, x, f, g):
        g[:, 0] = np.sum(self.W * x, axis=1) - self.C
        f[:, 0] = -np.sum(self.P * x, axis=1)


def create_random_knapsack_problem(n_items):
    P = np.random.randint(1, 100, size=n_items)
    W = np.random.randint(1, 100, size=n_items)
    C = np.sum(W) / 2
    problem = Knapsack(n_items, W, P, C)
    return problem



import numpy as np

from pymop.problem import Problem


class Ackley(Problem):
    def __init__(self, n_var=10, C1=20, C2=.2, C3=2 * np.pi):
        self.n_var = n_var
        self.c1 = C1
        self.c2 = C2
        self.c3 = C3
        self.n_constr = 0
        self.n_obj = 1
        self.func = self._evaluate
        self.xl = -32 * np.ones(self.n_var)
        self.xu = 32 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        part1 = -1. * self.c1 * np.exp(-1. * self.c2 * np.sqrt((1. / self.n_var) * np.sum(x * x, axis=1)))
        part2 = -1. * np.exp((1. / self.n_var) * np.sum(np.cos(self.c3 * x), axis=1))
        f[:, 0] = part1 + part2 + self.c1 + np.exp(1)



import numpy as np

from pymop.problem import Problem


class Rosenbrock(Problem):
    def __init__(self, n_var=2):
        Problem.__init__(self)
        self.n_var = n_var
        self.n_constr = 0
        self.n_obj = 1
        self.func = self._evaluate
        self.xl = -2.048 * np.ones(self.n_var)
        self.xu = 2.048 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        for i in range(x.shape[1] - 1):
            f[:, 0] += 100 * np.square((x[:, i + 1] - np.square(x[:, i]))) + np.square((1 - x[:, i]))


from setuptools import setup, find_packages



def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name="pymop",
    version="0.2.1",
    author="Julian Blank",
    author_email="blankjul@egr.msu.edu",
    description="Optimization Test Problems",
    long_description=readme(),
    url="https://github.com/msu-coinlab/pymop",
    license='MIT',
    keywords="optimization",
    install_requires=['numpy'],
    packages=find_packages(exclude=['tests', 'docs']),
    include_package_data=True,
)



import numpy as np

from pymop.problem import Problem


class Sphere(Problem):
    def __init__(self, n_var=10):
        self.n_var = n_var
        self.n_constr = 0
        self.n_obj = 1
        self.func = self._evaluate
        self.xl = 0 * np.ones(self.n_var)
        self.xu = 1 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        f[:, 0] = np.sum(np.square(x - 0.5), axis=1)



import numpy as np

from pymop.problem import Problem


class Kursawe(Problem):
    def __init__(self):
        Problem.__init__(self)
        self.n_var = 3
        self.n_constr = 0
        self.n_obj = 2
        self.func = self._evaluate
        self.xl = -5 * np.ones(self.n_var)
        self.xu = 5 * np.ones(self.n_var)

    def _evaluate(self, x, f):
        for i in range(2):
            f[:, 0] += -10 * np.exp(-0.2 * np.sqrt(np.square(x[:, i]) + np.square(x[:, i + 1])))
        f[:, 1] += np.sum(np.power(np.abs(x), 0.8) + 5 * np.sin(np.power(x, 3)), axis=1)


import numpy as np

from pymop.problem import Problem


class Truss2D(Problem):

    def __init__(self):
        Problem.__init__(self)
        self.n_var = 3
        self.n_constr = 1
        self.n_obj = 2
        self.func = self._evaluate

        self.Amax = 0.01
        self.Smax = 1e5

        self.xl = np.array([0.0, 0.0, 1.0])
        self.xu = np.array([self.Amax, self.Amax, 3.0])

    def _evaluate(self, x, f, g):

        # variable names for convenient access
        x1 = x[:, 0]
        x2 = x[:, 1]
        y = x[:, 2]

        # first objectives
        f[:, 0] = x1 * np.sqrt(16 + np.square(y)) + x2 * np.sqrt((1 + np.square(y)))

        # measure which are needed for the second objective
        sigma_ac = 20 * np.sqrt(16 + np.square(y)) / (y * x1)
        sigma_bc = 80 * np.sqrt(1 + np.square(y)) / (y * x2)

        # take the max
        f[:, 1] = np.max(np.column_stack((sigma_ac, sigma_bc)), axis=1)

        # define a constraint
        g[:, 0] = f[:, 1] - self.Smax
